#!/usr/bin/env python

import cgi, os, sys

sys.path.append("/var/lib/imip-agent")

import markup
import imip_store

getenv = os.environ.get
setenv = os.environ.__setitem__

class CGIEnvironment:

    "A CGI-compatible environment."

    def __init__(self):
        self.args = None
        self.method = None
        self.path = None
        self.path_info = None
        self.user = None

    def get_args(self):
        if self.args is None:
            if self.get_method() != "POST":
                setenv("QUERY_STRING", "")
            self.args = cgi.parse(keep_blank_values=True)
        return self.args

    def get_method(self):
        if self.method is None:
            self.method = getenv("REQUEST_METHOD") or "GET"
        return self.method

    def get_path(self):
        if self.path is None:
            self.path = getenv("SCRIPT_NAME") or ""
        return self.path

    def get_path_info(self):
        if self.path_info is None:
            self.path_info = getenv("PATH_INFO") or ""
        return self.path_info

    def get_user(self):
        if self.user is None:
            self.user = getenv("REMOTE_USER") or ""
        return self.user

    def get_output(self):
        return sys.stdout

    def get_url(self):
        path = self.get_path()
        path_info = self.get_path_info()
        return "%s%s" % (path.rstrip("/"), path_info)

class Manager:

    "A simple manager application."

    def __init__(self):
        self.store = imip_store.FileStore()
        self.env = CGIEnvironment()
        user = self.env.get_user()
        self.user = user and "mailto:%s" % user or None
        self.out = self.env.get_output()
        self.page = markup.page()
        self.encoding = "utf-8"

    def new_page(self, title):
        self.page.init(title=title, charset=self.encoding)

    def status(self, code, message):
        print >>self.out, "Status:", code, message

    def no_user(self):
        self.status(403, "Forbidden")
        self.new_page(title="Forbidden")
        self.page.p("You are not logged in and thus cannot access scheduling requests.")

    def show_requests(self):

        "Show requests for the current user."

        # NOTE: This list could be more informative, but it is envisaged that
        # NOTE: the requests would be visited directly anyway.

        self.new_page(title="Pending Requests")
        self.page.ul()

        requests = self.store.get_requests(self.user)
        for request in requests:
            self.page.li()
            self.page.a(request, href="%s/%s" % (self.env.get_url(), request))
            self.page.li.close()

        self.page.ul.close()

    def select_action(self):

        "Select the desired action and show the result."

        path_info = self.env.get_path_info().rstrip("/")
        if not path_info:
            self.show_requests()

    def show(self):

        "Interpret a request and show an appropriate response."

        if not self.user:
            self.no_user()
        else:
            self.select_action()

        print >>self.out, "Content-Type: text/html; charset=%s" % self.encoding
        print >>self.out
        self.out.write(unicode(self.page).encode(self.encoding))

if __name__ == "__main__":
    Manager().show()

# vim: tabstop=4 expandtab shiftwidth=4
